from vector.assistive_functions import get_normal, clamp_value, translate

class Color:
	def __get_rgb(self, *args) -> list[float]:
		args = args[0]
		number_of_args = len(args)

		if number_of_args == 0:
			return [0, 0, 0]  # no arguments
		elif number_of_args == 3:
			return list(args)  # x, y and z passed in
		elif number_of_args == 1:  # one argument
			arg_type = type(args[0])

			if arg_type is float or arg_type is int:  # single int or float argument
				return [args[0], args[0], args[0]]
			if arg_type is list or arg_type is tuple:
				if len(args[0]) == 1:
					return [args[0][0], args[0][0], args[0][0]]
				return [args[0][0], args[0][1], args[0][2]]  # single list argument
			if arg_type is Color:
				return [args[0].r, args[0].b, args[0].b]

		raise TypeError(f'Invalid Input: {args}')

	def __init__(self, *args) -> None:
		self.__elements = self.__get_rgb(args)

	#region Properties

	#---------------------- R
	@property
	def r(self):
		self.__elements[0] = clamp_value(self.__elements[0], 0, 255)
		return int(self.__elements[0])

	@r.setter
	def r(self, val):
		val = clamp_value(val, 0, 255)
		self.__elements[0] = val

	#---------------------- G
	@property
	def g(self):
		self.__elements[1] = clamp_value(self.__elements[1], 0, 255)
		return int(self.__elements[1])

	@g.setter
	def g(self, val):
		val = clamp_value(val, 0, 255)
		self.__elements[1] = val

	#---------------------- B
	@property
	def b(self):
		self.__elements[2] = clamp_value(self.__elements[2], 0, 255)
		return int(self.__elements[2])

	@b.setter
	def b(self, val):
		val = clamp_value(val, 0, 255)
		self.__elements[2] = val

	#---------------------- Hex

	@property
	def hex(self):
		return '#FFFFFF' #TODO Add Hex function

	#endregion

	#region Creation methods

	@staticmethod
	def random():
		return Color(get_normal() * 255, get_normal() * 255, get_normal() * 255)

	@staticmethod
	def zero():
		return Color()

	@staticmethod
	def from_hex(hex):
		c = []

		hex = hex.replace('#', '')
		for i in range(0, 6, 2):
			element = hex[i:i+2]
			c.append(int(element, 16))

		return Color(c)

	#endregion

	#region Conversion methods

	def as_hex(self):
		hexadecimal = ''
		for i in range(3):
			element = hex(self.__getitem__(i)).replace('0x', '').upper()
			if len(element) == 1:
				element = '0' + element
			hexadecimal += element

		return hexadecimal

	def as_unit(self):
		c = []
		for i in range(3):
			element = translate(self.__getitem__(i), 0, 255, 0, 1)
			c.append(element)

		return c

	#endregion

	#region General manipulation methods

	def get(self):
		return [int(self.r), int(self.g), int(self.b)]

	def set(self, *args):
		r, g, b = self.__get_rgb(args)
		self.__elements = r, g, b

	def copy(self):
		return Color(self.r, self.g, self.b)

	def clear(self):
		self.r = self.g = self.b = 0

	#endregion

	#region Dunder methods

	def __iadd__(self, *args):
		r, g, b = self.__get_rgb(args)
		self.r += r
		self.g += g
		self.b += b
		return self

	def __isub__(self, *args):
		r, g, b = self.__get_rgb(args)
		self.r -= r
		self.g -= g
		self.b -= b
		return self

	def __imul__(self, *args):
		r, g, b = self.__get_rgb(args)
		self.r *= r
		self.g *= g
		self.b *= b
		return self

	def __idiv__(self, *args):
		r, g, b = self.__get_rgb(args)
		self.r /= r
		self.g /= g
		self.b /= b
		return self

	def __add__(self, *args):
		r, g, b = self.__get_rgb(args)
		return Color(self.r + r, self.g + g, self.b + b)

	def __sub__(self, *args):
		r, g, b = self.__get_rgb(args)
		return Color(self.r - r, self.g - g, self.b - b)

	def __mul__(self, *args):
		r, g, b = self.__get_rgb(args)
		return Color(self.r * r, self.g * g, self.b * b)

	def __div__(self, *args):
		r, g, b = self.__get_rgb(args)
		return Color(self.r / r, self.g / g, self.b / b)

	def __getitem__(self, index):
		return int(self.__elements[index])

	def __repr__(self):
		return f'Color R: {self.r}, G: {self.g}, B: {self.b}'

	#endregion



from random import randint


def get_unit():
	return randint(-10000, 10000) / 10000


def get_normal():
	return randint(0, 10000) / 10000


def clamp_value(value, min, max):
	if value > max:
		return max

	if value < min:
		return min

	return value


def translate(value, leftMin, leftMax, rightMin, rightMax):
    # Figure out how 'wide' each range is
    leftSpan = leftMax - leftMin
    rightSpan = rightMax - rightMin

    # Convert the left range into a 0-1 range (float)
    valueScaled = float(value - leftMin) / float(leftSpan)

    # Convert the 0-1 range into a value in the right range.
    return rightMin + (valueScaled * rightSpan)



from math import sin, cos, atan2, sqrt
from random import randint
from vector.assistive_functions import get_normal, get_unit


class Vec2d:
	def __get_xy(self, *args) -> list[float]:
		args = args[0]
		number_of_args = len(args)

		if number_of_args == 0:
			return [0, 0]  # no arguments
		elif number_of_args == 2:
	   		x, y = args
	   		return [x, y]  # both x and y passed in
		elif number_of_args == 1:  # one argument
			arg_type = type(args[0])

			if arg_type is float or arg_type is int:  # single int or float argument
				return [args[0], args[0]]
			if arg_type is list or arg_type is tuple:
				if len(args[0]) == 1:
					return [args[0][0], args[0][0]]
				return [args[0][0], args[0][1]]  # single list argument
			if arg_type is Vec2d:
				return [args[0].x, args[0].y]

		raise TypeError(f'Invalid Input: {args}')

	def __init__(self, *args) -> None:
		self.__elements = self.__get_xy(args)


	#region Properties

	#---------------------- X
	@property
	def x(self):
		return self.__elements[0]
	@x.setter
	def x(self, val):
		self.__elements[0] = val

	#---------------------- Y
	@property
	def y(self):
		return self.__elements[1]
	@y.setter
	def y(self, val):
		self.__elements[1] = val

	#---------------------- W
	@property
	def w(self):
		return self.__elements[0]
	@w.setter
	def w(self, val):
		self.__elements[0] = val

	#---------------------- H
	@property
	def h(self):
		return self.__elements[1]
	@h.setter
	def h(self, val):
		self.__elements[1] = val

	#---------------------- Magnitude
	@property
	def length(self):
		return self.get_magnitude()


	#endregion


	#region Creation methods

	@staticmethod
	def random_unit():
		return Vec2d(get_unit(), get_unit())

	@staticmethod
	def random_pos():
		return Vec2d(get_normal(), get_normal())

	@staticmethod
	def from_angle(angle):
		return Vec2d(cos(angle), sin(angle))

	@staticmethod
	def zero():
		return Vec2d()

	#endregion


	#region General manipulation methods

	def get(self):
		return list(self.__elements)

	def get_int(self):
		return [int(self.x), int(self.y)]

	def set(self, *args):
		x, y = self.__get_xy(args)
		self.__elements = x, y

	def copy(self):
		return Vec2d(self.x, self.y)

	def clear(self):
		self.x = self.y = 0

	#endregion


	#region Mathematical manipulation methods

	def dist_sqrt(self, *args):
		x, y = self.__get_xy(args)
		return sqrt((self.x - x)**2 + (self.y - y)**2)

	def dist(self, *args):
		x, y = self.__get_xy(args)
		return (self.x - x)**2 + (self.y - y)**2

	def get_heading_angle(self):
		return atan2(self.x, self.y)

	def get_magnitude(self):
		return sqrt(self.x**2 + self.y**2)

	def normalise(self):
		mag = self.length
		if mag == 0 : return
		self.div(mag)

	def clamp(self, *args):
		max_x, max_y = self.__get_xy(args)

		if self.x > max_x : self.x = max_x
		if self.y > max_y : self.y = max_y

		if self.x < -max_x : self.x = -max_x
		if self.y < -max_y : self.y = -max_y

	def add(self, *args):
		x, y = self.__get_xy(args)
		self.x += x
		self.y += y

	def sub(self, *args):
		x, y = self.__get_xy(args)
		self.x -= x
		self.y -= y

	def mult(self, *args):
		x, y = self.__get_xy(args)
		self.x *= x
		self.y *= y

	def div(self, *args):
		x, y = self.__get_xy(args)
		self.x /= x
		self.y /= y

	def linear_interpolate(self, *args, t=0.5):
		x, y = self.__get_xy(args)

		x = self.x + t * (x - self.x)
		y = self.y + t * (y - self.y)

		self.set(x, y)

	def dot_product(self, *args):
		x, y = self.__get_xy(args)
		return sum([self.x * x, self.y * y])

	#endregion


	#region Dunder methods

	def __iadd__(self, *args):
		x, y = self.__get_xy(args)
		self.x += x ; self.y += y
		return self

	def __isub__(self, *args):
		x, y = self.__get_xy(args)
		self.x -= x ; self.y -= y
		return self

	def __imul__(self, *args):
		x, y = self.__get_xy(args)
		self.x *= x ; self.y *= y
		return self

	def __idiv__(self, *args):
		x, y = self.__get_xy(args)
		self.x /= x ; self.y /= y
		return self

	def __add__(self, *args):
		x, y = self.__get_xy(args)
		return Vec2d(self.x + x, self.y + y)

	def __sub__(self, *args):
		x, y = self.__get_xy(args)
		return Vec2d(self.x - x, self.y - y)

	def __mul__(self, *args):
		x, y = self.__get_xy(args)
		return Vec2d(self.x * x, self.y * y)

	def __div__(self, *args):
		x, y = self.__get_xy(args)
		return Vec2d(self.x / x, self.y / y)

	def __getitem__(self, index):
		return self.__elements[index]

	def __repr__(self):
		return f'vector X: {self.x}, Y: {self.y}'

	#endregion



